#pragma once

#include <map>

#include "aabb.hpp"
#include "assetptr.hpp"
#include "math.hpp"
#include "utility.hpp"

class GFXBuffer;
class GFXTexture;

class Texture : public Asset {
public:
    GFXTexture* handle = nullptr;
    int width = 0, height = 0;
};

class GFXPipeline;

enum class DataType {
    Vector3,
    Float,
    AssetTexture
};

struct MaterialProperty {
    std::string name;
    DataType type;

    MaterialProperty() {
        name = "";
        type = DataType::Vector3;
    }

    prism::float3 value;
    AssetPtr<Texture> value_tex;
    float float_value = 0.0f;
};

class Material : public Asset {
public:
    MaterialProperty colorProperty;
    MaterialProperty normalProperty;

    float metallic = 0.0f;
    float roughness = 0.5f;

    GFXPipeline* static_pipeline = nullptr;
    GFXPipeline* skinned_pipeline = nullptr;
    GFXPipeline* capture_pipeline = nullptr;

    std::map<int, AssetPtr<Texture>> bound_textures;
};

constexpr int max_weights_per_vertex = 4;

struct BoneVertexData {
    std::array<int, max_weights_per_vertex> ids;
    std::array<float, max_weights_per_vertex> weights;
};

struct Bone {
    size_t index = 0;

    std::string name;
    Matrix4x4 local_transform, final_transform;

    Bone* parent = nullptr;

    prism::float3 position;
    Quaternion rotation;
    prism::float3 scale;
};

class Mesh : public Asset {
public:
    // meshes are rendered in parts if we cannot batch it in one call, i.e. a
    // mesh with multiple materials with different textures, etc
    struct Part {
        std::string name;
        prism::aabb bounding_box;

        GFXBuffer* bone_batrix_buffer = nullptr;
        std::vector<Matrix4x4> offset_matrices;

        uint32_t index_offset = 0, index_count = 0;
        ;
        int32_t vertex_offset = 0;
        int32_t material_override = -1;
        std::string material_hint;
    };

    std::vector<Part> parts;
    std::vector<Bone> bones;
    Bone* root_bone = nullptr;

    // atributes
    GFXBuffer* position_buffer = nullptr;
    GFXBuffer* normal_buffer = nullptr;
    GFXBuffer* texture_coord_buffer = nullptr;
    GFXBuffer* tangent_buffer = nullptr;
    GFXBuffer* bitangent_buffer = nullptr;
    GFXBuffer* bone_buffer = nullptr;

    GFXBuffer* index_buffer = nullptr;

    Matrix4x4 global_inverse_transformation;

    uint32_t num_indices = 0;
};
